!> \file io_netcdf_GRISLI.f90
!! NetCDF Fortran 90 read/write interface
!<

!> \namespace io_netcdf_grisli
!! NetCDF Fortran 90 read/write interface
!! @note Use input/output functions provided by unidata                                                                    
!!                                                                               
!! @note Glossary of variables
!! @note varname   =  name of variable to trait
!! @note file,file_in,file_out      =  netcdf file name
!! @note tabvar    =  array containing values of the required variable
!! @note typevar   =  type of the variable to trait
!! @note dimIDs    =  array containing dimensions of variables  
!! @note dim'i'    =  dimensions of variables 
!! @note varid,varid_in,varid_out = variables identificator
!! @note ncid,ncid_in,ncid_out = netcdf file identificator
!! @note time      = time corresponding to the values to read/write  
!! @note level     = level corresponding to the values to read/write  
!! @note dimval    = value of the required dimension 
!! @note dimname   = name of dimension to retrieve
!!
!! http://my.unidata.ucar.edu/content/software/netcdf/docs/netcdf-f90/index.html  
!!                                                                               
!<

module io_netcdf_grisli
  !      
  use netcdf,only: nf90_open,nf90_close,nf90_nowrite,nf90_noerr
  use runparam,only:dirsource 
  !> \interface Read_Ncdf_var
  !! Interfaces of functions and subroutines to read netcdf variables 
  !!\author ...
  !!\date 2010
  !!
  !<
  integer :: ncdf_type = 0                    ! pour changer eventuellement de netcdf

  interface Read_Ncdf_var
     module procedure Read_Ncdf_var1d_Real,   &  ! Reading a 1d real variable
          Read_Ncdf_var2d_Real,               &  ! Reading a 2d real variable
          Read_Ncdf_var2d_Real_bis,           &  ! Reading a 2d real var with selection of data 
          Read_Ncdf_var3d_Real,               &  ! Reading a 3d real variable
          Read_Ncdf_var4d_Real,               &  ! Reading a 4d real variable
          Read_Ncdf_var1d_Int,                &  ! Reading a 1d integer variable
          Read_Ncdf_var2d_Int,                &  ! Reading a 2d integer variable
          Read_Ncdf_var3d_Int,                &  ! Reading a 3d integer variable
          Read_Ncdf_var4d_Int,                &  ! Reading a 4d integer variable
          Read_Ncdf_var3d_Real_t,             &  ! Reading a 2d variable depending on time 
          Read_Ncdf_var4d_Real_t,             &  ! Reading a 3d variable depending on time
          Read_Ncdf_var4d_Real_nt                ! Reading a 2d variable  at a time t and a level n 
  end interface
  !************************************************************************************************************

  !> \interface Write_Ncdf_var
  !! Interfaces of functions and  subroutine to write netcdf variables 
  !!\author ...
  !!\date 2010
  !!
  !<
  interface Write_Ncdf_var
     module procedure Write_Ncdf_var1d_Real,  &  ! Writing a 1d real (simple/double) variable
          Write_Ncdf_var2d_Real,              &  ! Writing a 2d real (simple/double) variable
          Write_Ncdf_var3d_Real,              &  ! Writing a 3d real (simple/double) variable
          Write_Ncdf_var4d_Real,              &  ! Writing a 4d real (simple/double) variable
          Write_Ncdf_var1d_Int,               &  ! Writing a 1d integer variable
          Write_Ncdf_var2d_Int,               &  ! Writing a 2d integer variable
          Write_Ncdf_var3d_Int,               &  ! Writing a 3d integer variable
          Write_Ncdf_var4d_Int,               &  ! Writing a 4d integer variable
          Write_Ncdf_var1d_Int_t,             &  ! Writing a 1d integer variable depending on time
          Write_Ncdf_var1d_Realbis_t,         &  ! Writing a 1d real (simple/double) variable depending on time
          Write_Ncdf_var1d_Real_t,            &  ! Writing a 1d real (simple/double) variable depending on time
          Write_Ncdf_var2d_Int_t,             &	 ! Writing a 2d integer variable depending on time	
          Write_Ncdf_var2d_Real_t,            &  ! Writing a 2d real (simple/double) variable depending on time
          Write_Ncdf_var3d_Real_t,            &  ! Writing a 3d real (simple/double) variable depending on time
          Write_Ncdf_var4d_Real_t,            &  ! Writing a 4d real (simple/double) variable depending on time
          Write_Ncdf_var4d_Real_nt               ! Writing a 2d variable  at a time t and a level n                                                     

  end interface
  !************************************************************************************************************* 

  !> \interface Copy_Ncdf_att
  !! Interfaces of functions and subroutines to copy netcdf variables 
  !!\author ...
  !!\date 2010
  !!
  !<
  interface Copy_Ncdf_att
     module procedure Copy_Ncdf_att_latlon,Copy_Ncdf_att_var
  end interface

contains

! subroutine pour lire netcdf_type
  subroutine lect_netcdf_type

    use runparam,only:dirsource

    implicit none
    ! open(22,file='../SOURCES/Fichiers-parametres/netcdf_type.dat')
    open(22,file=trim(dirsource)//'/Fichiers-parametres/netcdf_type.dat')
    read(22,'(i3)') ncdf_type
    close(22)
    write(6,*) ' lecture de ncdft_type',ncdf_type 
    return
  end subroutine lect_netcdf_type

  !> Subroutine to retrieve values of a given 1D real variable
  !! @param[in]  varname : name of variable to retrieve
  !! @param[in]  file    : netcdf file name                                
  !! @param[out] tabvar  : array containing values of the required variable
  !! @return tabvar

  subroutine Read_Ncdf_var1d_Real(varname,file,tabvar)

    use netcdf,only: nf90_open,nf90_close,nf90_nowrite,nf90_noerr,nf90_inq_varid, &
                     nf90_get_var,nf90_inquire_variable,nf90_inquire_dimension
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file
    Real*8, dimension(:), pointer :: tabvar
    !
    !local variables
    !
    Integer, dimension(1) :: dimID   
    Integer :: dim1                  
    Integer :: status,ncid           
    Integer :: varid             
    !
    status = nf90_open(file,NF90_NOWRITE,ncid)
          
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_varid(ncid,varname,varid)

    if (status/=nf90_noerr) then    
       write(*,*)"pb de nom de variable :", varname, file
       stop
    endif

!  recupere l'identite des dimensions de la variable
    status = nf90_inquire_variable(ncid,varid,dimids=dimID)

!  recupere la taille de la  dimension de la variable
    status = nf90_inquire_dimension(ncid,dimID(1),len=dim1)
    !                
    if(.not. associated(tabvar)) then
       Allocate(tabvar(dim1))  
    else
       if( any(shape(tabvar)/=(/dim1/)) ) then           
          deallocate(tabvar)   
          Allocate(tabvar(dim1))      
       endif
    endif
            
!  lit la variable
    status=nf90_get_var(ncid,varid,tabvar)     
      
! referme le fichier
    status = nf90_close(ncid)
    ! 
  end subroutine Read_Ncdf_var1d_Real



  !> Subroutine to retrieve values of a given 2D real variable
  !! @param[in]  varname : name of variable to retrieve
  !! @param[in]  file    : netcdf file name                                
  !! @param[out] tabvar  : array containing values of the required variable
  !! @return tabvar

  subroutine Read_Ncdf_var2d_Real(varname,file,tabvar)
    use netcdf,only: nf90_open,nf90_close,nf90_nowrite,nf90_noerr,nf90_inq_varid, &
                     nf90_get_var,nf90_inquire_variable,nf90_inquire_dimension
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file
    Real*8, dimension(:,:), pointer :: tabvar
    !local variables
    Integer, dimension(10) :: dimIDS
    Integer :: dim1,dim2
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_NOWRITE,ncid)
    !      
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_varid(ncid,varname,varid)
    if (status/=nf90_noerr) then
       write(*,*)"pb de nom de variable :", varname, file
       stop
    endif
    status=nf90_inquire_variable(ncid,varid,dimids=dimIDS)
    status=nf90_inquire_dimension(ncid,dimIDS(1),len=dim1)
    status=nf90_inquire_dimension(ncid,dimIDS(2),len=dim2)
    !                
    if(.not. associated(tabvar)) then
       Allocate(tabvar(dim1,dim2))  
    else
       if( any(shape(tabvar)/=(/dim1,dim2/)) ) then           
          tabvar => null()
          Allocate(tabvar(dim1,dim2))      
       endif
    endif
    !       
    status=nf90_get_var(ncid,varid,tabvar)     
    !     
    status = nf90_close(ncid)
    !     
  end subroutine Read_Ncdf_var2d_Real



  !> Subroutine to retrieve values of a given 2D real variable
  !! @param[in]  varname : name of variable to retrieve
  !! @param[in]  file    : netcdf file name                                
  !! @param[out] tabvar  : array containing values of the required variable
  !! @param[in]  strt    : array of integers specifiying the index in the  
  !!              variable from which the first (or only) of the  
  !!              data values will be read. The elements of start 
  !!              correspond, in order, to the variable's dimensions                                      
  !! @param[in]  cnt     : A vector of integers specifying the number of   
  !!               indices selected along each dimension.          
  !! @return tabvar


  subroutine Read_Ncdf_var2d_Real_bis(varname,file,tabvar,strt,cnt)
    use netcdf,only: nf90_open,nf90_close,nf90_nowrite,nf90_noerr,nf90_inq_varid, &
                     nf90_get_var,nf90_inquire_variable,nf90_inquire_dimension

    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file
    Real*8, dimension(:,:), pointer :: tabvar
    !local variables
    Integer, dimension(10) :: dimIDS
    Integer, dimension(2) :: strt,cnt
    Integer :: dim1,dim2
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_NOWRITE,ncid)
    !      
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    dim1 = cnt(1) 
    dim2 = cnt(2)
    !      
    status = nf90_inq_varid(ncid,varname,varid)
    if (status/=nf90_noerr) then
       write(*,*)"pb de nom de variable :", varname, file
       stop
    endif
    status=nf90_inquire_variable(ncid,varid,dimids=dimIDS)
    !                
    if(.not. associated(tabvar)) then
       Allocate(tabvar(dim1,dim2))  
    else
       if( any(shape(tabvar)/=(/dim1,dim2/)) ) then           
          deallocate(tabvar)   
          Allocate(tabvar(dim1,dim2))      
       endif
    endif
    !       
    status=nf90_get_var(ncid,varid,tabvar,start = strt,count = cnt)     
    !     
    status = nf90_close(ncid)
    !     
  end subroutine Read_Ncdf_var2d_Real_bis


  !> Subroutine to retrieve values of a given 3D real variable
  !! @param[in]  varname : name of variable to retrieve
  !! @param[in]  file    : netcdf file name                                
  !! @param[out] tabvar  : array containing values of the required variable
  !! @return tabvar
  subroutine Read_Ncdf_var3d_Real(varname,file,tabvar)
    use netcdf,only: nf90_open,nf90_close,nf90_nowrite,nf90_noerr,nf90_inq_varid, &
                     nf90_get_var,nf90_inquire_variable,nf90_inquire_dimension
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file
    Real*8, dimension(:,:,:), pointer :: tabvar
    !
    !local variables
    !
    Integer, dimension(10) :: dimIDS
    Integer :: dim1,dim2,dim3
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_NOWRITE,ncid)
    !      
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_varid(ncid,varname,varid)
    if (status/=nf90_noerr) then
       write(*,*)"pb de nom de variable :", varname, file
       stop
    endif
    status=nf90_inquire_variable(ncid,varid,dimids=dimIDS)
    status=nf90_inquire_dimension(ncid,dimIDS(1),len=dim1)
    status=nf90_inquire_dimension(ncid,dimIDS(2),len=dim2)  
    status=nf90_inquire_dimension(ncid,dimIDS(3),len=dim3)
    !                
    if(.not. associated(tabvar)) then
       Allocate(tabvar(dim1,dim2,dim3))  
    else
       if( any(shape(tabvar) /= (/dim1,dim2,dim3/)) ) then           
          deallocate(tabvar)   
          Allocate(tabvar(dim1,dim2,dim3))      
       endif
    endif
    !      
    status=nf90_get_var(ncid,varid,tabvar) 
    if (status/=nf90_noerr) then    
       write(*,*)"unable to retrieve netcdf variable : ",trim(varname)
       stop
    endif
    !     
    status = nf90_close(ncid)
    !     
  end subroutine Read_Ncdf_var3d_Real


  !> Subroutine to retrieve values of a given 4D real variable
  !! @param[in]  varname : name of variable to retrieve
  !! @param[in]  file    : netcdf file name                                
  !! @param[out] tabvar  : array containing values of the required variable
  !! @return tabvar
  subroutine Read_Ncdf_var4d_Real(varname,file,tabvar)
    use netcdf,only: nf90_open,nf90_close,nf90_nowrite,nf90_noerr,nf90_inq_varid, &
                     nf90_get_var,nf90_inquire_variable,nf90_inquire_dimension

    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file
    Real*8, dimension(:,:,:,:), pointer :: tabvar
    !
    !local variables
    !
    Integer, dimension(10) :: dimIDS
    Integer :: dim1,dim2,dim3,dim4
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_NOWRITE,ncid)
    !      
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_varid(ncid,varname,varid)
    if (status/=nf90_noerr) then
       write(*,*)"pb de nom de variable :", varname, file
       stop
    endif
    status=nf90_inquire_variable(ncid,varid,dimids=dimIDS)
    status=nf90_inquire_dimension(ncid,dimIDS(1),len=dim1)
    status=nf90_inquire_dimension(ncid,dimIDS(2),len=dim2)  
    status=nf90_inquire_dimension(ncid,dimIDS(3),len=dim3)   
    status=nf90_inquire_dimension(ncid,dimIDS(4),len=dim4)       
    !                
    if(.not. associated(tabvar)) then
       Allocate(tabvar(dim1,dim2,dim3,dim4))  
    else
       if( any(shape(tabvar) /= (/dim1,dim2,dim3,dim4/)) ) then           
          deallocate(tabvar)   
          Allocate(tabvar(dim1,dim2,dim3,dim4))      
       endif
    endif
    !  
    status=nf90_get_var(ncid,varid,tabvar)     
    !     
    status = nf90_close(ncid)
    !     
  end subroutine Read_Ncdf_var4d_Real

  !> Subroutine to retrieve values of a given 1D integer variable
  !! @param[in]  varname : name of variable to retrieve
  !! @param[in]  file    : netcdf file name                                
  !! @param[out] tabvar  : array containing values of the required variable
  !! @return tabvar
  subroutine Read_Ncdf_var1d_Int(varname,file,tabvar)
    use netcdf,only: nf90_open,nf90_close,nf90_nowrite,nf90_noerr,nf90_inq_varid, &
                     nf90_get_var,nf90_inquire_variable,nf90_inquire_dimension
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file
    Integer, dimension(:), pointer :: tabvar
    !
    !local variables
    !
    Integer,dimension(10) :: dimID
    Integer :: dim1
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_NOWRITE,ncid)
    !      
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_varid(ncid,varname,varid)
    if (status/=nf90_noerr) then
       write(*,*)"pb de nom de variable :", varname, file
       stop
    endif
    status=nf90_inquire_variable(ncid,varid,dimids=dimID)
    status=nf90_inquire_dimension(ncid,dimID(1),len=dim1)          
    !                
    if(.not. associated(tabvar)) then
       Allocate(tabvar(dim1))  
    else
       if( any(shape(tabvar) /= (/dim1/)) ) then           
          deallocate(tabvar)   
          Allocate(tabvar(dim1))      
       endif
    endif
    !  
    status=nf90_get_var(ncid,varid,tabvar)     
    !     
    status = nf90_close(ncid)
    !      
  end subroutine Read_Ncdf_var1d_Int


  !> Subroutine to retrieve values of a given 2D integer variable
  !! @param[in]  varname : name of variable to retrieve
  !! @param[in]  file    : netcdf file name                                
  !! @param[out] tabvar  : array containing values of the required variable
  !! @return tabvar
  subroutine Read_Ncdf_var2d_Int(varname,file,tabvar)
    use netcdf,only: nf90_open,nf90_close,nf90_nowrite,nf90_noerr,nf90_inq_varid, &
                     nf90_get_var,nf90_inquire_variable,nf90_inquire_dimension
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file
    Integer, dimension(:,:), pointer :: tabvar
    !local variables
    Integer, dimension(10) :: dimIDS
    Integer :: dim1,dim2
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_NOWRITE,ncid)
    !      
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_varid(ncid,varname,varid)
    if (status/=nf90_noerr) then
       write(*,*)"pb de nom de variable :", varname, file
       stop
    endif
    status=nf90_inquire_variable(ncid,varid,dimids=dimIDS)
    status=nf90_inquire_dimension(ncid,dimIDS(1),len=dim1)
    status=nf90_inquire_dimension(ncid,dimIDS(2),len=dim2)      
    !                
    if(.not. associated(tabvar)) then
       Allocate(tabvar(dim1,dim2))  
    else
       if( any(shape(tabvar) /= (/dim1,dim2/)) ) then           
          deallocate(tabvar)   
          Allocate(tabvar(dim1,dim2))      
       endif
    endif
    !  
    status=nf90_get_var(ncid,varid,tabvar)     
    !     
    status = nf90_close(ncid)
    !      
  end subroutine Read_Ncdf_var2d_Int

  !> Subroutine to retrieve values of a given 3D integer variable
  !! @param[in]  varname : name of variable to retrieve
  !! @param[in]  file    : netcdf file name                                
  !! @param[out] tabvar  : array containing values of the required variable
  !! @return tabvar
  subroutine Read_Ncdf_var3d_Int(varname,file,tabvar)
    use netcdf,only: nf90_open,nf90_close,nf90_nowrite,nf90_noerr,nf90_inq_varid, &
                     nf90_get_var,nf90_inquire_variable,nf90_inquire_dimension
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file
    Integer, dimension(:,:,:), pointer :: tabvar
    !
    !local variables
    !
    Integer, dimension(10) :: dimIDS
    Integer :: dim1,dim2,dim3
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_NOWRITE,ncid)
    !      
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_varid(ncid,varname,varid)
    if (status/=nf90_noerr) then
       write(*,*)"pb de nom de variable :", varname, file
       stop
    endif
    status=nf90_inquire_variable(ncid,varid,dimids=dimIDS)
    status=nf90_inquire_dimension(ncid,dimIDS(1),len=dim1)
    status=nf90_inquire_dimension(ncid,dimIDS(2),len=dim2)  
    status=nf90_inquire_dimension(ncid,dimIDS(3),len=dim3)          
    !                
    if(.not. associated(tabvar)) then
       Allocate(tabvar(dim1,dim2,dim3))  
    else
       if( any(shape(tabvar) /= (/dim1,dim2,dim3/)) ) then           
          deallocate(tabvar)   
          Allocate(tabvar(dim1,dim2,dim3))      
       endif
    endif
    ! 
    status=nf90_get_var(ncid,varid,tabvar)     
    !     
    status = nf90_close(ncid)
    !      
  end subroutine Read_Ncdf_var3d_Int

  !> Subroutine to retrieve values of a given 4D integer variable
  !! @param[in]  varname : name of variable to retrieve
  !! @param[in]  file    : netcdf file name                                
  !! @param[out] tabvar  : array containing values of the required variable
  !! @return tabvar

  subroutine Read_Ncdf_var4d_Int(varname,file,tabvar)
    use netcdf,only: nf90_open,nf90_close,nf90_nowrite,nf90_noerr,nf90_inq_varid, &
                     nf90_get_var,nf90_inquire_variable,nf90_inquire_dimension
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file
    Integer, dimension(:,:,:,:), pointer :: tabvar
    !
    !local variables
    !
    Integer, dimension(10) :: dimIDS
    Integer :: dim1,dim2,dim3,dim4
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_NOWRITE,ncid)
    !      
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_varid(ncid,varname,varid)
    if (status/=nf90_noerr) then
       write(*,*)"pb de nom de variable :", varname, file
       stop
    endif
    status=nf90_inquire_variable(ncid,varid,dimids=dimIDS)
    status=nf90_inquire_dimension(ncid,dimIDS(1),len=dim1)
    status=nf90_inquire_dimension(ncid,dimIDS(2),len=dim2)  
    status=nf90_inquire_dimension(ncid,dimIDS(3),len=dim3)   
    status=nf90_inquire_dimension(ncid,dimIDS(4),len=dim4)       
    !                
    if(.not. associated(tabvar)) then
       Allocate(tabvar(dim1,dim2,dim3,dim4))  
    else
       if( any(shape(tabvar) /= (/dim1,dim2,dim3,dim4/)) ) then           
          deallocate(tabvar)   
          Allocate(tabvar(dim1,dim2,dim3,dim4))      
       endif
    endif
    !  
    status=nf90_get_var(ncid,varid,tabvar)     
    !     
    status = nf90_close(ncid)
    !     
  end subroutine Read_Ncdf_var4d_Int


  !> Subroutine to retrieve values of a given 4D real variable
  !! @param[in]  varname : name of variable to retrieve
  !! @param[in]  file    : netcdf file name                                
  !! @param[out] tabvar  : array containing values of the required variable
  !! @param[in]  time    : time corresponding to the values to read
  !! @param[in]  level   : level corresponding to the values to read
  !! @return tabvar

  subroutine Read_Ncdf_var4d_Real_nt(varname,file,tabvar,time,level)     
    use netcdf,only: nf90_open,nf90_close,nf90_nowrite,nf90_noerr,nf90_inq_varid, &
                     nf90_get_var,nf90_inquire_variable,nf90_inquire_dimension
    implicit none
    !       
    Character(*),intent(in) :: varname,file
    Integer,intent(in) :: time,level
    Real*8, dimension(:,:,:,:), pointer :: tabvar
    !
    !local variables
    !
    Integer, dimension(4) :: dimIDS
    Integer :: dim1,dim2
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_NOWRITE,ncid)
    !      
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_varid(ncid,varname,varid)
    if (status/=nf90_noerr) then
       write(*,*)"pb de nom de variable :", varname, file
       stop
    endif        
    status=nf90_inquire_variable(ncid,varid,dimids=dimIDS)
    status=nf90_inquire_dimension(ncid,dimIDS(1),len=dim1)
    status=nf90_inquire_dimension(ncid,dimIDS(2),len=dim2)
    !
    if(.not. associated(tabvar)) then
       Allocate(tabvar(dim1,dim2,1,1))  
    else
       if( any(shape(tabvar) /= (/dim1,dim2,1,1/)) ) then           
          deallocate(tabvar)   
          Allocate(tabvar(dim1,dim2,1,1))      
       endif
    endif
    !
    status=nf90_get_var(ncid,varid,tabvar,start=(/1,1,level,time/))
    !                                             
    if (status/=nf90_noerr) then    
       write(*,*)"unable to retrieve netcdf variable : ",trim(varname)
       stop
    endif
    !     
    status = nf90_close(ncid)
    !     
  end subroutine Read_Ncdf_var4d_Real_nt


  !> Subroutine to retrieve values of a given 3D real variable withe time dependent
  !! @param[in]  varname : name of variable to retrieve
  !! @param[in]  file    : netcdf file name                                
  !! @param[out] tabvar  : array containing values of the required variable
  !! @param[in]  time    : time corresponding to the values to read
  !! @return tabvar
  subroutine Read_Ncdf_var4d_Real_t(varname,file,tabvar,time)    
    use netcdf,only: nf90_open,nf90_close,nf90_nowrite,nf90_noerr,nf90_inq_varid, &
                     nf90_get_var,nf90_inquire_variable,nf90_inquire_dimension
    implicit none
    !       
    Character(*),intent(in) :: varname,file
    Integer,intent(in) :: time
    Real*8, dimension(:,:,:,:), pointer :: tabvar
    !
    !local variables
    !
    Integer, dimension(4) :: dimIDS
    Integer :: dim1,dim2,dim3
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_NOWRITE,ncid)
    !      
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_varid(ncid,varname,varid)
    if (status/=nf90_noerr) then
       write(*,*)"pb de nom de variable :", varname, file
       stop
    endif        
    status=nf90_inquire_variable(ncid,varid,dimids=dimIDS)
    status=nf90_inquire_dimension(ncid,dimIDS(1),len=dim1)
    status=nf90_inquire_dimension(ncid,dimIDS(2),len=dim2)
    status=nf90_inquire_dimension(ncid,dimIDS(3),len=dim3)
    !
    if(.not. associated(tabvar)) Allocate(tabvar(dim1,dim2,dim3,1))  
    status=nf90_get_var(ncid,varid,tabvar,start=(/1,1,1,time/))

    if (status/=nf90_noerr) then    
       write(*,*)"unable to retrieve netcdf variable : ",trim(varname)
       stop
    endif
    !     
    status = nf90_close(ncid)
    !     
  end subroutine Read_Ncdf_var4d_Real_t

  !> Subroutine to read a 3D real variable in a given file for time t
  !! @param[in]  varname : name of variable to retrieve
  !! @param[in]  file    : netcdf file name                                
  !! @param[out] tabvar  : array containing values of the required variable
  !! @param[in]  time    : time corresponding to the values to read
  !! @return tabvar


  subroutine Read_Ncdf_var3d_Real_t(varname,file,tabvar,time)   
    use netcdf,only: nf90_open,nf90_close,nf90_nowrite,nf90_noerr,nf90_inq_varid, &
                     nf90_get_var,nf90_inquire_variable,nf90_inquire_dimension
    implicit none
    !       
    Character(*),intent(in) :: varname,file
    Integer,intent(in) :: time
    Real*8, dimension(:,:,:), pointer :: tabvar
    !
    !local variables
    !
    Integer, dimension(3) :: dimIDS
    Integer :: dim1,dim2
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_NOWRITE,ncid)
    !      
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_varid(ncid,varname,varid)
    !        
    status=nf90_inquire_variable(ncid,varid,dimids=dimIDS)
    status=nf90_inquire_dimension(ncid,dimIDS(1),len=dim1)
    status=nf90_inquire_dimension(ncid,dimIDS(2),len=dim2)
    !
    if(.not. associated(tabvar)) then
       Allocate(tabvar(dim1,dim2,1))  
    else
       if( any(shape(tabvar) /= (/dim1,dim2,1/)) ) then           
          deallocate(tabvar)   
          Allocate(tabvar(dim1,dim2,1))      
       endif
    endif

    status=nf90_get_var(ncid,varid,tabvar,start=(/1,1,time/))

    if (status/=nf90_noerr) then    
       write(*,*)"unable to retrieve netcdf variable : ",trim(varname)
       stop
    endif
    !     
    status = nf90_close(ncid)
    !     
  end subroutine Read_Ncdf_var3d_Real_t

  !> Subroutine to write a 1D real variable in a given file 
  !!@param[in]     varname : name of variable to store                            
  !!@param[in]     dimname : name of dimensions of the given variable        
  !!@param[in]     file    : netcdf file name                                
  !!@param[in]     tabvar  : values of the variable to write                 
  !!@param[in]     typevar : type of the variable to write                   

  subroutine Write_Ncdf_var1d_Real(varname,dimname,file,tabvar,typevar)
    use netcdf,only: nf90_open,nf90_close,nf90_write,nf90_noerr,nf90_inq_varid, nf90_double, &
                     nf90_float,nf90_def_var,nf90_put_var,nf90_inq_dimid,nf90_enddef,nf90_redef
    !     
    implicit none
    !       
    Character(*),intent(in) :: varname,file,dimname,typevar
    Real*8, dimension(:), pointer :: tabvar
    !
    ! local variables
    !
    Integer :: dimid
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_WRITE,ncid)       
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    

    status = nf90_inq_dimid(ncid,dimname, dimid)

    status = nf90_inq_varid(ncid,varname,varid)
!    write(6,*) 'varid status',status,'no error :',nf90_NoErr,' varid :',varid,'  varname: ',varname
!    write(6,*)  'erreur',trim(nf90_strerror(status))

    status = nf90_redef(ncid)

    select case(TRIM(typevar))
    case('double')
       status = nf90_def_var(ncid,varname,nf90_double,(/dimid/),varid)

    case('float')
       status = nf90_def_var(ncid,varname,nf90_float,(/dimid/),varid)  
    end select
 
    status = nf90_enddef(ncid)
    status = nf90_put_var(ncid,varid,tabvar) 

    !     
    status = nf90_close(ncid)
    ! 
  end subroutine Write_Ncdf_var1d_Real

  !> Subroutine to write a 2D real variable in a given file 
  !!@param[in]     varname : name of variable to store                            
  !!@param[in]     dimname : name of dimensions of the given variable        
  !!@param[in]     file    : netcdf file name                                
  !!@param[in]     tabvar  : values of the variable to write                 
  !!@param[in]     typevar : type of the variable to write                   
  

  subroutine Write_Ncdf_var2d_Real(varname,dimname,file,tabvar,typevar)
    use netcdf,only: nf90_open,nf90_close,nf90_write,nf90_noerr,nf90_inq_varid, &
                     nf90_def_var,nf90_put_var,nf90_inq_dimid,nf90_enddef,nf90_redef
    !      
    !      implicit none
    !       
    Character(*),intent(in) :: varname,file,typevar
    Character(*), dimension(2) :: dimname
    Real*8, dimension(:,:), pointer :: tabvar
    !
    ! local variables
    !
    Integer :: dimid1,dimid2
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_WRITE,ncid)       
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_dimid(ncid,dimname(1), dimid1)
    status = nf90_inq_dimid(ncid,dimname(2), dimid2)
    
    status = nf90_inq_varid(ncid,varname,varid)
    status = nf90_redef(ncid)
    
    select case(TRIM(typevar))
    case('double')
       status = nf90_def_var(ncid,varname,nf90_double,     &
            (/dimid1,dimid2/),varid)
    case('float')
       status = nf90_def_var(ncid,varname,nf90_float,     &
            (/dimid1,dimid2/),varid)     
    end select
    !
    status = nf90_enddef(ncid)
    status = nf90_put_var(ncid,varid,tabvar)     
    !     
    status = nf90_close(ncid)
    ! 
  end subroutine Write_Ncdf_var2d_Real

  !> Subroutine to write a 3D real variable in a given file 
  !!@param[in]     varname : name of variable to store                            
  !!@param[in]     dimname : name of dimensions of the given variable        
  !!@param[in]     file    : netcdf file name                                
  !!@param[in]     tabvar  : values of the variable to write                 
  !!@param[in]     typevar : type of the variable to write                   

  subroutine Write_Ncdf_var3d_Real(varname,dimname,file,tabvar,typevar)
    use netcdf,only: nf90_open,nf90_close,nf90_write,nf90_noerr,nf90_inq_varid,nf90_double, &
                     nf90_float,nf90_put_var,nf90_inq_dimid,nf90_redef,nf90_def_var,nf90_enddef
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file,typevar
    Character(*),dimension(3),intent(in) :: dimname
    Real*8, dimension(:,:,:), pointer :: tabvar
    !
    ! local variables
    !
    Integer :: dimid1,dimid2,dimid3
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_WRITE,ncid)       
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_dimid(ncid,dimname(1), dimid1)
    status = nf90_inq_dimid(ncid,dimname(2), dimid2)
    status = nf90_inq_dimid(ncid,dimname(3), dimid3)
    status = nf90_inq_varid(ncid,varname,varid)
    status = nf90_redef(ncid)

    select case(TRIM(typevar))
    case('double')
       status = nf90_def_var(ncid,varname,nf90_double,     &
            (/dimid1,dimid2,dimid3/),varid)
    case('float')
       status = nf90_def_var(ncid,varname,nf90_float,     &
            (/dimid1,dimid2,dimid3/),varid)    
    end select
    !
    status = nf90_enddef(ncid)
    status = nf90_put_var(ncid,varid,tabvar)     
    !     
    status = nf90_close(ncid)
    ! 
  end subroutine Write_Ncdf_var3d_Real

  !> Subroutine to write a 4D real variable in a given file 
  !!@param[in]     varname : name of variable to store                            
  !!@param[in]     dimname : name of dimensions of the given variable        
  !!@param[in]     file    : netcdf file name                                
  !!@param[in]     tabvar  : values of the variable to write                 
  !!@param[in]     typevar : type of the variable to write                   

  subroutine Write_Ncdf_var4d_Real(varname,dimname,file,tabvar,typevar)
    use netcdf,only: nf90_open,nf90_close,nf90_write,nf90_noerr,nf90_inq_varid,nf90_double, &
                     nf90_float,nf90_put_var,nf90_def_var,nf90_inq_dimid,nf90_redef,nf90_enddef
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file,typevar
    Character(*),dimension(4),intent(in) :: dimname
    Real*8, dimension(:,:,:,:), pointer :: tabvar
    !
    ! local variables
    !
    Integer :: dimid1,dimid2,dimid3,dimid4
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_WRITE,ncid)       
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !
    status = nf90_inq_varid(ncid,varname,varid)
    !      
    if(status/=nf90_noerr) then
       !     
       status = nf90_inq_dimid(ncid,dimname(1), dimid1)
       status = nf90_inq_dimid(ncid,dimname(2), dimid2)
       status = nf90_inq_dimid(ncid,dimname(3), dimid3)
       status = nf90_inq_dimid(ncid,dimname(4), dimid4)
       status = nf90_redef(ncid)
       !      
       select case(TRIM(typevar))
       case('double')
          status = nf90_def_var(ncid,varname,nf90_double,     &
               (/dimid1,dimid2,dimid3,dimid4/),varid)
       case('float')
          status = nf90_def_var(ncid,varname,nf90_float,     &
               (/dimid1,dimid2,dimid3,dimid4/),varid)   
       end select
       !
       status = nf90_enddef(ncid)
    endif
    !         
    status = nf90_put_var(ncid,varid,tabvar)     
    !     
    status = nf90_close(ncid)
    ! 
  end subroutine Write_Ncdf_var4d_Real


  !> Subroutine to write a 1D integer variable in a given file 
  !!@param[in]     varname : name of variable to store                            
  !!@param[in]     dimname : name of dimensions of the given variable        
  !!@param[in]     file    : netcdf file name                                
  !!@param[in]     tabvar  : values of the variable to write                              


  subroutine Write_Ncdf_var1d_Int(varname,dimname,file,tabvar)
    use netcdf,only: nf90_open,nf90_close,nf90_write,nf90_noerr,nf90_inq_varid,nf90_int, &
                     nf90_def_var,nf90_put_var,nf90_inq_dimid,nf90_enddef,nf90_redef
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file,dimname
    Integer, dimension(:), pointer :: tabvar
    !
    ! local variables
    !
    Integer :: dimid
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_WRITE,ncid)       
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_dimid(ncid,dimname, dimid)
    status = nf90_inq_varid(ncid,varname,varid)
    status = nf90_redef(ncid)
    status = nf90_def_var(ncid,varname,nf90_int,(/dimid/),varid)
    status = nf90_enddef(ncid)
    status = nf90_put_var(ncid,varid,tabvar)     
    !     
    status = nf90_close(ncid)
    ! 
  end subroutine Write_Ncdf_var1d_Int

  !> Subroutine to write a 2D integer variable in a given file 
  !!@param[in]     varname : name of variable to store                            
  !!@param[in]     dimname : name of dimensions of the given variable        
  !!@param[in]     file    : netcdf file name                                
  !!@param[in]    tabvar  : values of the variable to write                              

  subroutine Write_Ncdf_var2d_Int(varname,dimname,file,tabvar)
    use netcdf,only: nf90_open,nf90_close,nf90_write,nf90_noerr,nf90_inq_varid,nf90_int, &
                     nf90_def_var,nf90_put_var,nf90_inq_dimid,nf90_enddef,nf90_redef
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file
    Character(*), dimension(2) :: dimname
    Integer, dimension(:,:), pointer :: tabvar
    !
    ! local variables
    !
    Integer :: dimid1,dimid2
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_WRITE,ncid)       
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_dimid(ncid,dimname(1), dimid1)
    status = nf90_inq_dimid(ncid,dimname(2), dimid2)
    status = nf90_inq_varid(ncid,varname,varid)
    status = nf90_redef(ncid)
    status = nf90_def_var(ncid,varname,nf90_int,     &
         (/dimid1,dimid2/),varid)
    status = nf90_enddef(ncid)
    status = nf90_put_var(ncid,varid,tabvar)     
    !     
    status = nf90_close(ncid)
    ! 
  end subroutine Write_Ncdf_var2d_Int

  !> Subroutine to write a 3D integer variable in a given file 
  !!@param[in]     varname : name of variable to store                            
  !!@param[in]     dimname : name of dimensions of the given variable        
  !!@param[in]     file    : netcdf file name                                
  !!@param[in]     tabvar  : values of the variable to write                              

  subroutine Write_Ncdf_var3d_Int(varname,dimname,file,tabvar)
    use netcdf,only: nf90_open,nf90_close,nf90_write,nf90_noerr,nf90_inq_varid,nf90_int, &
                     nf90_def_var,nf90_put_var,nf90_inq_dimid,nf90_enddef,nf90_redef
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file
    Character(*),dimension(3),intent(in) :: dimname
    Integer, dimension(:,:,:), pointer :: tabvar
    !
    ! local variables
    !
    Integer :: dimid1,dimid2,dimid3
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_WRITE,ncid)       
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_dimid(ncid,dimname(1), dimid1)
    status = nf90_inq_dimid(ncid,dimname(2), dimid2)
    status = nf90_inq_dimid(ncid,dimname(3), dimid3)
    status = nf90_inq_varid(ncid,varname,varid)
    status = nf90_redef(ncid)
    status = nf90_def_var(ncid,varname,nf90_int,     &
         (/dimid1,dimid2,dimid3/),varid)
    status = nf90_enddef(ncid)
    status = nf90_put_var(ncid,varid,tabvar)     
    !     
    status = nf90_close(ncid)
    ! 
  end subroutine Write_Ncdf_var3d_Int

  !> Subroutine to write a 4D integer variable in a given file 
  !!@param[in]     varname : name of variable to store                            
  !!@param[in]     dimname : name of dimensions of the given variable        
  !!@param[in]     file    : netcdf file name                                
  !!@param[in]    tabvar  : values of the variable to write                              


  subroutine Write_Ncdf_var4d_Int(varname,dimname,file,tabvar)
    use netcdf,only: nf90_open,nf90_close,nf90_write,nf90_noerr,nf90_inq_varid,nf90_int, &
                     nf90_def_var,nf90_put_var,nf90_inq_dimid,nf90_enddef,nf90_redef
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file
    Character(*),dimension(4),intent(in) :: dimname
    Integer, dimension(:,:,:,:), pointer :: tabvar
    !
    ! local variables
    !
    Integer :: dimid1,dimid2,dimid3,dimid4
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_WRITE,ncid)       
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_dimid(ncid,dimname(1), dimid1)
    status = nf90_inq_dimid(ncid,dimname(2), dimid2)
    status = nf90_inq_dimid(ncid,dimname(3), dimid3)
    status = nf90_inq_dimid(ncid,dimname(4), dimid4)
    status = nf90_inq_varid(ncid,varname,varid)
    status = nf90_redef(ncid)
    status = nf90_def_var(ncid,varname,nf90_int,     &
         (/dimid1,dimid2,dimid3,dimid4/),varid)
    status = nf90_enddef(ncid)
    status = nf90_put_var(ncid,varid,tabvar)     
    !     
    status = nf90_close(ncid)
    ! 
  end subroutine Write_Ncdf_var4d_Int

  !> Subroutine to write a 2D real variable with time dependent in a given file 
  !!@param[in]     varname : name of variable to store                            
  !!@param[in]     dimname : name of dimensions of the given variable        
  !!@param[in]     file    : netcdf file name                                
  !!@param[in]     tabvar  : values of the variable to write 
  !!@param[in]     time    : time corresponding to the values to write
  !!@param[inout]  idef    : 
  !!@param[in]     typevar : type of the variable to write
  !!@return idef
  subroutine Write_Ncdf_var2d_Real_t(varname,dimname,file,tabvar,time,idef,typevar)
    use netcdf,only: nf90_open,nf90_close,nf90_write,nf90_noerr,nf90_inq_varid,nf90_double, &
                     nf90_float,nf90_def_var,nf90_put_var,nf90_inq_dimid,nf90_enddef,nf90_redef
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file,typevar
    Character(*),dimension(3),intent(in) :: dimname
    Integer :: time
    Integer,intent(inout) :: idef  
    Real*8, dimension(:,:), pointer :: tabvar
    !
    ! local variables
    !
    Integer :: dimid1,dimid2,dimid3
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_WRITE,ncid)       
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    if(time==1 .OR. idef==0) then
       !      
       status = nf90_inq_dimid(ncid,dimname(1), dimid1)
       status = nf90_inq_dimid(ncid,dimname(2), dimid2)
       status = nf90_inq_dimid(ncid,dimname(3), dimid3)
       status = nf90_redef(ncid)
       !      
       select case(TRIM(typevar))
       case('double')
          status = nf90_def_var(ncid,varname,nf90_double,     &
               (/dimid1,dimid2,dimid3/),varid) 
       case('float')
          status = nf90_def_var(ncid,varname,nf90_float,     &
               (/dimid1,dimid2,dimid3/),varid)    
       end select
       !
       status = nf90_enddef(ncid)
       idef = 1
    else
       status = nf90_inq_varid(ncid, varname, varid)
    endif
    !      
    status = nf90_put_var(ncid,varid,tabvar,start=(/1,1,time/))
    if (status/=nf90_noerr) then    
       write(*,*)"unable to store variable ",varname, &
            " in file ",file
       stop
    endif
    !     
    status = nf90_close(ncid)
    ! 
  end subroutine Write_Ncdf_var2d_Real_t

  !> Subroutine to write a 2D integer variable with time dependent in a given file 
  !!@param[in]     varname : name of variable to store                            
  !!@param[in]     dimname : name of dimensions of the given variable        
  !!@param[in]     file    : netcdf file name                                
  !!@param[in]     tabvar  : values of the variable to write 
  !!@param[in]     time    : time corresponding to the values to write
  !!@param[inout]  idef    :
  !!@return idef

  subroutine Write_Ncdf_var2d_Int_t(varname,dimname,file,tabvar,time,idef)
    use netcdf,only: nf90_open,nf90_close,nf90_write,nf90_noerr,nf90_inq_varid,nf90_int, &
                     nf90_def_var,nf90_put_var,nf90_inq_dimid,nf90_enddef,nf90_redef
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file
    Character(*),dimension(3),intent(in) :: dimname
    Integer :: time
    Integer,intent(inout) :: idef
    Integer, dimension(:,:), pointer :: tabvar
    !
    ! local variables
    !
    Integer :: dimid1,dimid2,dimid3
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_WRITE,ncid)       
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    if(time==1 .OR. idef==0 ) then
       !      
       status = nf90_inq_dimid(ncid,dimname(1), dimid1)
       status = nf90_inq_dimid(ncid,dimname(2), dimid2)
       status = nf90_inq_dimid(ncid,dimname(3), dimid3)
       status = nf90_redef(ncid)
       !      
       status = nf90_def_var(ncid,varname,nf90_int,     &
            (/dimid1,dimid2,dimid3/),varid) 
       !
       status = nf90_enddef(ncid)
       idef=1
    else
       status = nf90_inq_varid(ncid, varname, varid)
    endif
    !      
    status = nf90_put_var(ncid,varid,tabvar,start=(/1,1,time/))
    if (status/=nf90_noerr) then    
       write(*,*)"unable to store variable ",varname, &
            " in file ",file
       stop
    endif
    !     
    status = nf90_close(ncid)
    ! 
  end subroutine Write_Ncdf_var2d_Int_t


  !> Subroutine to write a 1D real variable with time dependent in a given file 
  !!@param[in]     varname : name of variable to store                            
  !!@param[in]     dimname : name of dimensions of the given variable        
  !!@param[in]     file    : netcdf file name                                
  !!@param[in]     tabvar  : values of the variable to write 
  !!@param[in]     time    : time corresponding to the values to write
  !!@param[inou]   idef    :
  !!@param[in]     typevar : type of the variable to write
  !!@return idef


  subroutine Write_Ncdf_var1d_Real_t(varname,dimname,file,tabvar,time,idef,typevar)
    use netcdf,only: nf90_open,nf90_close,nf90_write,nf90_noerr,nf90_inq_varid,nf90_double, &
                     nf90_float,nf90_def_var,nf90_put_var,nf90_inq_dimid,nf90_enddef,nf90_redef
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file,typevar
    Character(*),dimension(2),intent(in) :: dimname
    Integer :: time
    Integer,intent(inout) :: idef
    Real*8, dimension(:) :: tabvar
    !
    ! local variables
    !
    Integer :: dimid1,dimid2
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_WRITE,ncid)       
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    if(time==1 .OR. idef==0) then
       !      
       status = nf90_inq_dimid(ncid,dimname(1), dimid1)
       status = nf90_inq_dimid(ncid,dimname(2), dimid2)
       status = nf90_redef(ncid)
       !      
       select case(TRIM(typevar))
       case('double')
          status = nf90_def_var(ncid,varname,nf90_double,     &
               (/dimid1,dimid2/),varid) 
       case('float')
          status = nf90_def_var(ncid,varname,nf90_float,     &
               (/dimid1,dimid2/),varid)    
       end select
       !
       status = nf90_enddef(ncid)
       idef=1
    else
       status = nf90_inq_varid(ncid, varname, varid)
    endif
    !      

    status = nf90_put_var(ncid,varid,tabvar,start=(/1,time/))
    if (status/=nf90_noerr) then    
       write(*,*)"unable to store variable ",varname, &
            " in file ",file
       stop
    endif
    !     
    status = nf90_close(ncid)
    ! 
  end subroutine Write_Ncdf_var1d_Real_t

  !> Subroutine to write a 1D real variable with time dependent in a given file 
  !!@param[in]     varname : name of variable to store                            
  !!@param[in]     dimname : name of dimensions of the given variable        
  !!@param[in]     file    : netcdf file name                                
  !!@param[in]     tabvar  : values of the variable to write 
  !!@param[in]     time    : time corresponding to the values to write
  !!@param[inout]  idef    :
  !!@param[in]     typevar : type of the variable to write
  !!@return idef

  subroutine Write_Ncdf_var1d_Realbis_t(varname,dimname,file,tabvar,time,idef,typevar)
    use netcdf,only: nf90_open,nf90_close,nf90_write,nf90_noerr,nf90_inq_varid,nf90_double, &
                     nf90_float,nf90_def_var,nf90_put_var,nf90_inq_dimid,nf90_enddef,nf90_redef
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file,typevar
    Character(*),intent(in) :: dimname
    Integer :: time
    Integer,intent(inout) :: idef
    Real*8, dimension(:), pointer :: tabvar
    !
    ! local variables
    !
    Integer :: dimid1
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_WRITE,ncid)       
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    if(time==1 .OR. idef==0) then
       !      
       status = nf90_inq_dimid(ncid,dimname, dimid1)
       status = nf90_redef(ncid)
       !      
       select case(TRIM(typevar))
       case('double')
          status = nf90_def_var(ncid,varname,nf90_double,     &
               (/dimid1/),varid) 
       case('float')
          status = nf90_def_var(ncid,varname,nf90_float,     &
               (/dimid1/),varid)    
       end select
       !
       status = nf90_enddef(ncid)
       idef=1
    else
       status = nf90_inq_varid(ncid, varname, varid)
    endif
    !     

    status = nf90_put_var(ncid,varid,tabvar,start=(/time/))
    if (status/=nf90_noerr) then    
       write(*,*)"unable to store variable ",varname, &
            " in file ",file
       stop
    endif
    !     
    status = nf90_close(ncid)
    ! 
  end subroutine Write_Ncdf_var1d_Realbis_t


  !> Subroutine to write a 1D integer variable with time dependent in a given file 
  !!@param[in]     varname : name of variable to store                            
  !!@param[in]     dimname : name of dimensions of the given variable        
  !!@param[in]     file    : netcdf file name                                
  !!@param[in]     tabvar  : values of the variable to write 
  !!@param[in]     time    : time corresponding to the values to write
  !!@param[inout]  idef    : 
  !!@param[in]     typevar : type of the variable to write
  !!@return idef
  subroutine Write_Ncdf_var1d_Int_t(varname,dimname,file,tabvar,time,idef,typevar)
    use netcdf,only: nf90_open,nf90_close,nf90_write,nf90_noerr,nf90_inq_varid,nf90_double, &
                     nf90_float,nf90_def_var,nf90_put_var,nf90_inq_dimid,nf90_enddef,nf90_redef
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file,typevar
    Character(*),dimension(2),intent(in) :: dimname
    Integer :: time
    Integer,intent(inout) :: idef
    !Integer, dimension(:), pointer :: tabvar
    Integer, dimension(:) :: tabvar
    !
    ! local variables
    !
    Integer :: dimid1,dimid2
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_WRITE,ncid)       
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    if(time==1 .OR. idef==0) then
       !      
       status = nf90_inq_dimid(ncid,dimname(1), dimid1)
       status = nf90_inq_dimid(ncid,dimname(2), dimid2)
       status = nf90_redef(ncid)

       !      

       status = nf90_def_var(ncid,varname,nf90_double,     &
            (/dimid1,dimid2/),varid) 

       !
       status = nf90_enddef(ncid)
       idef=1
    else
       status = nf90_inq_varid(ncid, varname, varid)
    endif
    !      
    status = nf90_put_var(ncid,varid,tabvar,start=(/1,time/))
    if (status/=nf90_noerr) then    
       write(*,*)"unable to store variable ",varname, &
            " in file ",file
       stop
    endif
    !     
    status = nf90_close(ncid)
    ! 
  end subroutine Write_Ncdf_var1d_Int_t

  !> Subroutine to write a 2D real variable with time dependent in a given file 
  !!@param[in]     varname : name of variable to store                            
  !!@param[in]     dimname : name of dimensions of the given variable        
  !!@param[in]     file    : netcdf file name                                
  !!@param[in]     tabvar  : values of the variable to write 
  !!@param[in]     time    : time corresponding to the values to write
  !!@param[inout]  idef    :
  !!@param[in]     typevar : type of the variable to write
  !!@return idef
  subroutine Write_Ncdf_var3d_Real_t(varname,dimname,file,tabvar,time,idef,typevar)
    use netcdf,only: nf90_open,nf90_close,nf90_write,nf90_noerr,nf90_inq_varid,nf90_double, &
                     nf90_float,nf90_def_var,nf90_put_var,nf90_inq_dimid,nf90_enddef,nf90_redef
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file,typevar
    Character(*),dimension(4),intent(in) :: dimname
    Integer :: time
    Integer,intent(inout) :: idef
    Real*8, dimension(:,:,:), pointer :: tabvar
    !
    ! local variables
    !
    Integer :: dimid1,dimid2,dimid3,dimid4
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_WRITE,ncid)       
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    if(time==1 .OR. idef==0) then
       !      
       status = nf90_inq_dimid(ncid,dimname(1), dimid1)
       status = nf90_inq_dimid(ncid,dimname(2), dimid2)
       status = nf90_inq_dimid(ncid,dimname(3), dimid3)
       status = nf90_inq_dimid(ncid,dimname(4), dimid4)
       status = nf90_redef(ncid)
       !      
       select case(TRIM(typevar))
       case('double')
          status = nf90_def_var(ncid,varname,nf90_double,     &
               (/dimid1,dimid2,dimid3,dimid4/),varid) 
       case('float')
          status = nf90_def_var(ncid,varname,nf90_float,     &
               (/dimid1,dimid2,dimid3,dimid4/),varid)    
       end select
       !
       status = nf90_enddef(ncid)
       idef=1
    else
       status = nf90_inq_varid(ncid, varname, varid)
    endif
    !      
    status = nf90_put_var(ncid,varid,tabvar,start=(/1,1,1,time/))
    if (status/=nf90_noerr) then    
       write(*,*)"unable to store variable ",varname, &
            " in file ",file
       stop
    endif
    !     
    status = nf90_close(ncid)
    ! 
  end subroutine Write_Ncdf_var3d_Real_t

  !> Subroutine to write a 3D real variable with time dependent in a given file 
  !!@param[in]     varname : name of variable to store                            
  !!@param[in]     dimname : name of dimensions of the given variable        
  !!@param[in]     file    : netcdf file name                                
  !!@param[in]     tabvar  : values of the variable to write 
  !!@param[in]     time    : time corresponding to the values to write
  !!@param[inout]  idef    :
  !!@param[in]     typevar : type of the variable to write
  !!@return idef
  subroutine Write_Ncdf_var4d_Real_t(varname,dimname,file,tabvar,time,idef,typevar)
    use netcdf,only: nf90_open,nf90_close,nf90_write,nf90_noerr,nf90_inq_varid,nf90_double, &
                     nf90_float,nf90_def_var,nf90_put_var,nf90_inq_dimid,nf90_enddef,nf90_redef
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file,typevar
    Character(*),dimension(4),intent(in) :: dimname
    Integer :: time
    Integer,intent(inout) :: idef
    Real*8, dimension(:,:,:,:), pointer :: tabvar
    !
    ! local variables
    !
    Integer :: dimid1,dimid2,dimid3,dimid4
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_WRITE,ncid)       
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !      
    if(time==1 .OR. idef==0) then
       !      
       status = nf90_inq_dimid(ncid,dimname(1), dimid1)
       status = nf90_inq_dimid(ncid,dimname(2), dimid2)
       status = nf90_inq_dimid(ncid,dimname(3), dimid3)
       status = nf90_inq_dimid(ncid,dimname(4), dimid4)
       status = nf90_redef(ncid)
       !      
       select case(TRIM(typevar))
       case('double')
          status = nf90_def_var(ncid,TRIM(varname),nf90_double,     &
               (/dimid1,dimid2,dimid3,dimid4/),varid)  
       case('float')
          status = nf90_def_var(ncid,TRIM(varname),nf90_float,     &
               (/dimid1,dimid2,dimid3,dimid4/),varid)    
       end select
       !
       status = nf90_enddef(ncid)
       idef=1
    else
       status = nf90_inq_varid(ncid, varname, varid)
    endif
    !    
    status = nf90_put_var(ncid,varid,tabvar,start=(/1,1,1,time/))
    if (status/=nf90_noerr) then    
       write(*,*)"unable to store variable ",varname, &
            " in file ",file
       stop
    endif
    !    
    status = nf90_close(ncid)
    ! 
  end subroutine Write_Ncdf_var4d_Real_t


  !> Subroutine to write a 3D real variable with time dependent in a given file 
  !!@param[in]     varname : name of variable to store                            
  !!@param[in]     dimname : name of dimensions of the given variable        
  !!@param[in]     file    : netcdf file name                                
  !!@param[in]     tabvar  : values of the variable to write 
  !!@param[in]     time    : time corresponding to the values to write
  !!@param[in]     level   : level corresponding to the values to read
  !!@param[inout]  idef    :
  !!@param[in]     typevar : type of the variable to write
  !!@return idef

  subroutine Write_Ncdf_var4d_Real_nt(varname,dimname,file,tabvar,time,level,idef,typevar)
    use netcdf,only: nf90_open,nf90_close,nf90_write,nf90_noerr,nf90_inq_varid,nf90_double, &
                     nf90_float,nf90_def_var,nf90_put_var,nf90_inq_dimid,nf90_enddef,nf90_redef
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file,typevar
    Character(*),dimension(4),intent(in) :: dimname
    Integer :: time,level
    Integer,intent(inout) :: idef
    Real*8, dimension(:,:,:,:), pointer :: tabvar
    !
    ! local variables
    !
    Integer :: dimid1,dimid2,dimid3,dimid4
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_WRITE,ncid)       
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !      
    if((time==1.and.level==1) .OR. idef==0 ) then
       !      
       status = nf90_inq_dimid(ncid,dimname(1), dimid1)
       status = nf90_inq_dimid(ncid,dimname(2), dimid2)
       status = nf90_inq_dimid(ncid,dimname(3), dimid3)
       status = nf90_inq_dimid(ncid,dimname(4), dimid4)
       status = nf90_redef(ncid)
       !      
       select case(TRIM(typevar))
       case('double')
          status = nf90_def_var(ncid,TRIM(varname),nf90_double,     &
               (/dimid1,dimid2,dimid3,dimid4/),varid)  
       case('float')
          status = nf90_def_var(ncid,TRIM(varname),nf90_float,     &
               (/dimid1,dimid2,dimid3,dimid4/),varid)    
       end select
       !
       status = nf90_enddef(ncid)
       idef=1
    else
       status = nf90_inq_varid(ncid, varname, varid)
    endif
    !    
    status = nf90_put_var(ncid,varid,tabvar,start=(/1,1,level,time/))
    if (status/=nf90_noerr) then    
       write(*,*)"unable to store variable ",varname, &
            " in file ",file
       stop
    endif
    !    
    status = nf90_close(ncid)
    ! 
  end subroutine Write_Ncdf_var4d_Real_nt

  !> Subroutine  to retrieve names of all variables included in a given file
  !!@param[in]     filename    : netcdf file name                    
  !!@param[out]    tabvarname  : array containing various variables names       
  !!@return tabvarname

  subroutine Read_Ncdf_VarName(filename,tabvarname)
    use netcdf,only: nf90_open,nf90_write,nf90_noerr,nf90_inquire,nf90_inquire_variable
    !      
    Character(*),intent(in) :: filename
    Character*20,dimension(:),pointer :: tabvarname
    Integer :: nDimensions,nVariables
    Integer :: nAttributes,unlimitedDimId,i
    Integer :: ncid,status
    !     
    status = nf90_open(filename,NF90_NOWRITE,ncid)
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",filename
       stop
    endif
    !      
    status = nf90_inquire(ncid,nDimensions,nVariables,nAttributes, &
         unlimitedDimId) 
    !
    Allocate(tabvarname(nVariables))
    !
    Do i=1,nVariables
       status = nf90_inquire_variable(ncid,i,tabvarname(i))
    End do

  end subroutine Read_Ncdf_Varname


  !> Subroutine copy a netcdf varaiable  included in a given file to an other file                 
  !!@param[in]    varname  : array containing various variables names       
  !!@param[in]    filein   : netcdf entry file name   
  !!@param[in]    fileout  : netcdf result file name
  !!@return 

  subroutine Copy_Ncdf_att_var(varname,filein,fileout)
    use netcdf,only: nf90_open,nf90_close,nf90_nowrite,nf90_noerr,nf90_inq_varid, &
                     nf90_write,nf90_copy_att,nf90_enddef,nf90_redef
    !      
    Character(*),intent(in) :: filein,fileout
    Character(*),intent(in) :: varname
    Integer :: ncid_in,ncid_out,status,varid_in,varid_out
    !     
    status = nf90_open(filein,NF90_NOWRITE,ncid_in)
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open input netcdf file : ",filein
       stop
    endif
    !                                                                             
    status = nf90_open(fileout,NF90_WRITE,ncid_out)
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open output netcdf file : ",fileout
       stop
    endif
    !  
    status = nf90_inq_varid(ncid_in,varname,varid_in)
    status = nf90_inq_varid(ncid_out,varname,varid_out)
    !
    status = nf90_redef(ncid_out)
    !      
    status = nf90_copy_att(ncid_in,varid_in,'units',ncid_out,varid_out)
    status = nf90_copy_att(ncid_in,varid_in,'valid_min',ncid_out,varid_out)
    status = nf90_copy_att(ncid_in,varid_in,'valid_max',ncid_out,varid_out) 
    status = nf90_copy_att(ncid_in,varid_in,'long_name',ncid_out,varid_out)
    status = nf90_copy_att(ncid_in,varid_in,'calendar',ncid_out,varid_out)
    status = nf90_copy_att(ncid_in,varid_in,'title',ncid_out,varid_out)     
    status = nf90_copy_att(ncid_in,varid_in,'time_origin',ncid_out,varid_out) 
    status = nf90_copy_att(ncid_in,varid_in,'positive',ncid_out,varid_out)                 
    status = nf90_copy_att(ncid_in,varid_in,'tstep_sec',ncid_out,varid_out)          
    status = nf90_copy_att(ncid_in,varid_in,'nav_model',ncid_out,varid_out)   
    status = nf90_copy_att(ncid_in,varid_in,'Minvalue=',ncid_out,varid_out)
    status = nf90_copy_att(ncid_in,varid_in,'Maxvalue=',ncid_out,varid_out)  
    status = nf90_copy_att(ncid_in,varid_in,'short_name',ncid_out,varid_out)
    status = nf90_copy_att(ncid_in,varid_in,'online_operation',ncid_out,varid_out)
    status = nf90_copy_att(ncid_in,varid_in,'axis',ncid_out,varid_out)            
    status = nf90_copy_att(ncid_in,varid_in,'interval_operation',ncid_out,varid_out)
    status = nf90_copy_att(ncid_in,varid_in,'interval_write',ncid_out,varid_out) 
    status = nf90_copy_att(ncid_in,varid_in,'associate',ncid_out,varid_out)
    status = nf90_copy_att(ncid_in,varid_in,'actual_range',ncid_out,varid_out) 
    status = nf90_copy_att(ncid_in,varid_in,'longitude',ncid_out,varid_out)
    status = nf90_copy_att(ncid_in,varid_in,'latitude',ncid_out,varid_out)
    status = nf90_copy_att(ncid_in,varid_in,'scale_factor',ncid_out,varid_out)
    status = nf90_copy_att(ncid_in,varid_in,'add_offset',ncid_out,varid_out)
    status = nf90_copy_att(ncid_in,varid_in,'missing_value',ncid_out,varid_out) 
    !      
    status = nf90_enddef(ncid_out)  
    !
    status = nf90_close(ncid_in)
    status = nf90_close(ncid_out)
    ! 
  end subroutine Copy_Ncdf_att_var

  !> Subroutine copy a netcdf varaiable  included in a given file to an other file at latlon                 
  !!@param[in]    varname  : array containing various variables names       
  !!@param[in]    filein   : netcdf entry file name   
  !!@param[in]    fileout  : netcdf result file name
  !!@param[in]    min      : 
  !!@param[in]    max      :
  !!@return 


  subroutine Copy_Ncdf_att_latlon(varname,filein,fileout,min,max)
    use netcdf,only: nf90_open,nf90_close,nf90_nowrite,nf90_noerr,nf90_inq_varid, &
                     nf90_write,nf90_put_att,nf90_copy_att,nf90_enddef,nf90_redef

    !      
    Character(*),intent(in) :: filein,fileout
    Character(*),intent(in) :: varname
    real*8 :: min,max
    Integer :: ncid_in,ncid_out,status,varid_in,varid_out
    !     
    status = nf90_open(filein,NF90_NOWRITE,ncid_in)
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",filein
       stop
    endif
    !                                                                             
    status = nf90_open(fileout,NF90_WRITE,ncid_out)
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",fileout
       stop
    endif
    !  
    status = nf90_inq_varid(ncid_in,varname,varid_in)
    status = nf90_inq_varid(ncid_out,varname,varid_out)
    !
    status = nf90_redef(ncid_out)
    !      
    select case (varname)
       !      
    case('nav_lon')     
       status = nf90_copy_att(ncid_in,varid_in,'units',ncid_out,varid_out) 
       status = nf90_put_att(ncid_out,varid_out,'valid_min',real(min,4))
       status = nf90_put_att(ncid_out,varid_out,'valid_max',real(max,4))
       status = nf90_copy_att(ncid_in,varid_in,'long_name',ncid_out,varid_out)
       status = nf90_copy_att(ncid_in,varid_in,'nav_model',ncid_out,varid_out)
       status = nf90_copy_att(ncid_in,varid_in,'title',ncid_out,varid_out)
       !
    case('nav_lat')
       status = nf90_copy_att(ncid_in,varid_in,'units',ncid_out,varid_out) 
       status = nf90_put_att(ncid_out,varid_out,'valid_min',real(min,4))
       status = nf90_put_att(ncid_out,varid_out,'valid_max',real(max,4))
       status = nf90_copy_att(ncid_in,varid_in,'long_name',ncid_out,varid_out)
       status = nf90_copy_att(ncid_in,varid_in,'nav_model',ncid_out,varid_out)
       status = nf90_copy_att(ncid_in,varid_in,'title',ncid_out,varid_out) 
       !
    end select
    !      
    status = nf90_enddef(ncid_out)  
    !
    status = nf90_close(ncid_in)
    status = nf90_close(ncid_out)
  end subroutine Copy_Ncdf_att_latlon

  !> Subroutine to retrieve value of a given dimension              
  !!@param[in]    dimname : name of dimension to retrieve     
  !!@param[in]    file    : netcdf file name       
  !!@param[out]   dimval  : value of the required dimension
  !!@return       dimval

  subroutine Read_Ncdf_dim(dimname,file,dimval)   
    use netcdf,only: nf90_open,nf90_close,nf90_nowrite,nf90_noerr,nf90_inq_varid, &
                     nf90_inq_dimid,nf90_inquire_dimension
    !      
    implicit none
    !           
    Character(*),intent(in) :: dimname,file    
    Integer    :: dimval
    !            
    ! local variables
    !
    integer ncid,status,dimid
    !     
    status = nf90_open(file,NF90_NOWRITE,ncid)
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_dimid(ncid,dimname,dimid)
    status = nf90_inquire_dimension(ncid,dimid,len=dimval)                                                                
    !      
    status = nf90_close(ncid)
    !      
  end subroutine Read_Ncdf_dim

  !> Subroutine   to write a dimension in a given file           
  !!@param[in]    dimname : name of dimension to retrieve     
  !!@param[in]    file    : netcdf file name       
  !!@param[out]   dimval  : value of the required dimension
  !!@return       dimval

  subroutine Write_Ncdf_dim(dimname,file,dimval)   
    use netcdf,only: nf90_open,nf90_close,nf90_write,nf90_noerr,nf90_inq_varid, &
                     nf90_enddef,nf90_redef,nf90_unlimited,nf90_def_dim

    !      
    implicit none
    !           
    Character(*),intent(in) :: dimname,file    
    Integer    :: dimval
    !      
    ! local variables
    !
    integer ncid,status,dimid
    !     
    status = nf90_open(file,NF90_WRITE,ncid)
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_redef(ncid)
    If(dimval.eq.0) then
       status = nf90_def_dim(ncid,dimname,nf90_unlimited,dimid)       
    Else
       status = nf90_def_dim(ncid,dimname,dimval,dimid)  
    End If
    status = nf90_enddef(ncid)
    !      
    status = nf90_close(ncid)
    !      
  end subroutine Write_Ncdf_dim

  !> Function   to get the dimension of a given vriable in a given file           
  !!@param[in]    varname : name of variable to retrieve dimension     
  !!@param[in]    filename    : netcdf file name       
  !!@return       

  Integer function Get_NbDims( varname , filename )
    use netcdf,only: nf90_open,nf90_nowrite,nf90_noerr,nf90_inq_varid, &
                     nf90_inquire_variable

    !
    Character(*),intent(in) :: varname,filename
    Integer :: status,ncid,varid    
    !      
    status = nf90_open(TRIM(filename),NF90_NOWRITE,ncid)
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",TRIM(filename)
       stop
    endif
    status = nf90_inq_varid(ncid,TRIM(varname),varid)     
    status = nf90_inquire_variable(ncid, varid , ndims = Get_NbDims)
    !
    return
    !
  end function Get_NbDims

 
  !> Function   to test exitence dimension in a given file           
  !!@param[in]    varname : name of variable to retrieve dimension     
  !!@param[in]    filename    : netcdf file name       
  !!@return       

  !   function Get_NbDims_Existence
  !**************************************************************          
  !
  Logical function Dims_Existence( dimname , filename )
    use netcdf,only: nf90_open,nf90_nowrite,nf90_noerr,nf90_inq_dimid

    !
    Character(*),intent(in) :: dimname,filename
    Integer :: status,ncid,dimid    
    !      
    status = nf90_open(TRIM(filename),NF90_NOWRITE,ncid)
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",TRIM(filename)
       stop
    endif
    status = nf90_inq_dimid(ncid,dimname,dimid)
    !      
    if (status/=nf90_noerr) then
       Dims_Existence = .false.
    else
       Dims_Existence = .true.
    endif
    !
    return
    !
  end function Dims_Existence
  !
  !**************************************************************
  !**************************************************************

end module io_netcdf_grisli






